from elasticai.creator.vhdl.translator.abstract.layers.linear_1d_translatable import (
    Linear1dTranslatable,
    Linear1dTranslationArgs,
)
from elasticai.creator.vhdl.translator.abstract.layers.lstm_translatable import (
    LSTMTranslatable,
    LSTMTranslationArgs,
)
