from elasticai.creator.vhdl.translator.pytorch.build_functions.linear_1d_build_function import (
    build_linear_1d,
)
from elasticai.creator.vhdl.translator.pytorch.build_functions.lstm_build_function import (
    build_lstm,
)
